library veerne_kit;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'controllers/button_controller.dart';

class Input extends StatelessWidget {
  final TextEditingController controller;
  final String placeholder;
  final IconData icon;
  final bool isPassword;
  final bool isNumber;
  final Color backgroundColor;
  final Function onChange;
  final Color textColor;
  final Color boxShadowColor;

  Input(
      {@required this.controller,
      @required this.icon,
      @required this.placeholder,
      this.isPassword = false,
      this.isNumber = false,
      this.backgroundColor = Colors.white,
      this.boxShadowColor = Colors.black12,
      this.textColor = Colors.black,
      this.onChange(String text)});

  @override
  Widget build(BuildContext context) => Container(
        padding: EdgeInsets.only(top: 5, bottom: 5, left: 15, right: 15),
        margin: EdgeInsets.all(5),
        decoration: BoxDecoration(
            color: backgroundColor,
            borderRadius: BorderRadius.circular(15),
            boxShadow: [
              BoxShadow(
                  color: this.boxShadowColor.withOpacity(.2),
                  offset: Offset(.5, 8),
                  blurRadius: 10,
                  spreadRadius: 1)
            ]),
        child: TextField(
          cursorColor: Colors.black,
          controller: controller,
          obscureText: isPassword,
          style: TextStyle(color: Colors.black),
          maxLength: 600,
          onChanged: (this.onChange != null ? this.onChange : null),
          keyboardType: this.isNumber
              ? TextInputType.numberWithOptions(decimal: false)
              : TextInputType.text,
          decoration: InputDecoration(
              icon: Icon(icon, color: this.textColor),
              border: InputBorder.none,
              labelText: placeholder,
              counterText: "",
              labelStyle: TextStyle(color: Colors.black)),
        ),
      );
}

class Button extends StatefulWidget {
  final String title;
  final Function(ButtonController) onTap;
  final EdgeInsets padding;
  final Color color;
  final Color textColor;

  Button({
    @required this.title,
    @required this.onTap,
    this.color,
    this.textColor = Colors.black,
    this.padding = const EdgeInsets.all(1),
  });

  @override
  _ButtonState createState() => _ButtonState();
}

class _ButtonState extends State<Button> {
  bool isLoading = false;

  void showLoading() => setState(() => isLoading = true);

  void hideLoading() => setState(() => isLoading = false);

  Widget renderContent() {
    if (isLoading) {
      return SizedBox(
        height: MediaQuery.of(context).size.height * .02,
        width: MediaQuery.of(context).size.height * .02,
        child: CircularProgressIndicator(
          backgroundColor: Colors.white,
        ),
      );
    }

    return Text(widget.title,
        style: TextStyle(color: widget.textColor, fontWeight: FontWeight.bold));
  }

  void showAlert(String message) {
    showModalBottomSheet(
        context: context,
        builder: (context) {
          return Center(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Icon(Icons.info_outline, color: Colors.black, size: 40),
                TitleItem(
                  message,
                  overflow: TextOverflow.fade,
                  textAlign: TextAlign.center,
                ),
                Divider(),
                FloatingActionButton.extended(
                  onPressed: () => RoutingService.goBack(context),
                  label: Text("Cerrar"),
                  backgroundColor: Colors.redAccent,
                ),
              ],
            ),
          );
        });
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.only(top: 3, left: 10, right: 10, bottom: 5),
      child: CupertinoButton(
        borderRadius: BorderRadius.circular(15),
        onPressed: () => this.widget.onTap(ButtonController(
            showLoading: showLoading,
            hideLoading: hideLoading,
            showAlert: showAlert)),
        color: widget.color != null
            ? widget.color
            : Color.fromRGBO(50, 152, 230, 1),
        child: Container(
          alignment: Alignment.center,
          padding: this.widget.padding,
          child: Center(child: renderContent()),
        ),
      ),
    );
  }
}

// ignore: must_be_immutable
class SkeletonText extends StatefulWidget {
  final double height;
  final double width;
  Color colors;

  SkeletonText(
      {Key key, this.height = 20, this.width = 200, this.colors = Colors.black})
      : super(key: key);

  createState() => SkeletonState();
}

class SkeletonState extends State<SkeletonText>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  Animation gradientPosition;

  @override
  void initState() {
    _controller = AnimationController(
        duration: Duration(milliseconds: 1500), vsync: this);
    gradientPosition = Tween<double>(
      begin: -3,
      end: 10,
    ).animate(
      CurvedAnimation(parent: _controller, curve: Curves.linear),
    )..addListener(() {
        setState(() {});
      });

    _controller.repeat();
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: widget.width,
      height: widget.height,
      margin: EdgeInsets.only(top: 5, bottom: 5),
      decoration: BoxDecoration(
          gradient: LinearGradient(
              begin: Alignment(gradientPosition.value, 0),
              end: Alignment(-1, 0),
              colors: [
                widget.colors.withOpacity(.05),
                widget.colors.withOpacity(.1),
                widget.colors.withOpacity(.05)
              ]),
          borderRadius: BorderRadius.circular(25)),
    );
  }
}

class TitleItem extends StatelessWidget {
  final double fontSize;
  final Color color;
  final TextAlign textAlign;
  final String text;
  final bool isTitle;
  final TextOverflow overflow;
  final bool isUnderline;

  TitleItem(this.text,
      {this.fontSize = 35,
      this.textAlign = TextAlign.left,
      this.color = Colors.black,
      this.isTitle = false,
      this.overflow = TextOverflow.fade,
      this.isUnderline = false});

  @override
  Widget build(BuildContext context) => Text(
        this.isTitle ? (this.text).toUpperCase() : this.text,
        style: TextStyle(
            color: color,
            decoration: isUnderline ? TextDecoration.underline : null,
            fontSize: fontSize,
            fontWeight: FontWeight.w900,
            letterSpacing: this.isTitle ? 3 : 0),
        textAlign: textAlign,
        overflow: this.overflow,
      );
}

class RoutingService {
  static Future goTo(BuildContext context,
      {@required Widget route,
      bool isFullScreen = true,
      Function onBack(dynamic value)}) async {
    return await Navigator.of(context)
        .push(CupertinoPageRoute(
            fullscreenDialog: isFullScreen, builder: (context) => route))
        .then((dynamic _data) => onBack != null ? onBack(_data) : null);
  }

  static replacementRoute(BuildContext context, {@required Widget route}) {
    return Navigator.pushReplacement(
      context,
      CupertinoPageRoute(
          fullscreenDialog: true,
          builder: (context) => WillPopScope(
                onWillPop: () async => false,
                child: route,
              )),
    );
  }

  static goBack(BuildContext context, {dynamic handle}) =>
      Navigator.pop(context, handle);
}
